% This code tests the performance of S3SCOMP on the COIL100.
% The code generates results in Table 2 of the paper
% "Stochastic Sparse Subspace Clustering", CVPR 20.


% Instructions for running the code:
% - Download code for computing clustering accuracy. Go to
% http://www.cad.zju.edu.cn/home/dengcai/Data/Clustering.html and download
% bestMap.m and Hungarian.m. Alternatively, you can use your own function
% by redefining the function evalAccuracy.m
% - Download databases.
%   = Coil-100: we use the data provided at
%   http://www.cad.zju.edu.cn/home/dengcai/Data/MLData.html
% - vl_feat toolbox. Download and install the vl_feat toolbox from
%   http://www.vlfeat.org/. The code uses the functions vl_kdtreebuild, 
%   vl_kdtreequery and vl_kmeans from this toolbox. Alternatively, one can
%   also use these functionalities provided with Matlab. 
% - Run. 
%% Setting
clear
buildRepresentation = @(data,dic,ind_1,lambda,C_final) S3COMP_C(data,dic,ind_1,lambda,C_final,3, 1e-6);
%% Load data
addpath('toolbox')
addpath('vlfeat-0.9.21')
run('vlfeat-0.9.21/toolbox/vl_setup')

load COIL100.mat fea gnd
nExperiment = 10;
data = double(fea');
label = gnd;   
nCluster = length(unique(label));
N = length(label); 
data = cnormalize_inplace(data);
%% Test begin
% timearray1=zeros(1,T+1);
% timearray2=zeros(1,nExperiment);
delta=0.1;  

T=15;
lambda=0.6;
% CoreNum=10;
% parpool ('local',CoreNum);
maxiter=3;
results = zeros(nExperiment, 5); %results
results_conn=zeros(nExperiment,nCluster+4);

for iExperiment = 1:nExperiment
    zz = RandStream('mt19937ar','Seed',iExperiment);
    RandStream.setGlobalStream(zz);
    mask = randperm(N);
    X = data(:, mask);
    s = label(mask);

    %for iExperiment = 1:nExperiment  
    C=cell(1,T+1);
    count=zeros(1,N);
    idxcell=cell(1,T+1);
    for t=1:T
        a=rand(1,N);
        a=a>delta;
        idx=find(a>0);
        count(idx)=count(idx)+1;
        idxcell{t}=idx;
    end
    if all(count)
        T_all=T;
    else
        id1=find(count<1);
        a=rand(1,N);
        a=a>delta;
        idx=find(a>0);
        idx=union(idx,id1);
        count(idx)=count(idx)+1;
        idxcell{T+1}=idx;
        T_all=T+1;
    end
    C_final=sparse(N,N);
    time1=0;
    time2=0;
    for iter=1:maxiter
        for t=1:T_all
            tic
            C_final_t=C_final(idxcell{t},:);
            C{t}=buildRepresentation(X,X(:,idxcell{t})*(1/(1-delta)),idxcell{t},lambda,C_final_t);
            time_tmp=toc;
            time2=max(time_tmp,time2);
        end
        tic
        R=sparse(N,N);
        for t=1:T_all
            R=R+C{t};
        end
        C_final=R/T_all;
        time_tmp=toc;
        time1=time1+time_tmp+time2;
    end
    tic
    C_final(1:N+1:end) = 0;
    A = abs(C_final) + abs(C_final)';
    groups = SpectralClustering(A, nCluster, 'Eig_Solver', 'eigs','Kmeans_Solver', 'vl_kmeans');  
    time_tmp=toc;
    time1=time1+time_tmp;
    
     % Evaluation
       % Evaluation
    [perc, vec] = evalSSR_perc( C_final, s );
    ssr = evalSSR_error( C_final, s );   
    [conn,conn_min,conn_max,conn_median,conn_mean] = evalConn_all( A, s);
%     conn = evalConn_mean( A, s);
    accr  = evalAccuracy(s, groups);
    results_conn(iExperiment,:)=[conn,conn_min,conn_max,conn_median,conn_mean];
    % output
    dataformat = '%d-th experiment: perc = %f, ssr = %f, conn_mean = %f, accr = %f\n';
    dataValue = [iExperiment, perc, ssr, conn_mean, accr];
    fprintf(dataformat, dataValue);
    % record
    results(iExperiment, :) = dataValue;
end
% output
dataValue = mean(results, 1);
dataValue_conn=mean(results_conn,1);
fprintf('\nAverage: perc = %f, ssr = %f, conn_mean = %f, accr = %f\n', dataValue(2:end));