% This code tests the performance of S3COMP-C on the MNIST digit database.
% The code generates results in Table 4 of the paper
% "Stochastic Sparse Subspace Clustering", CVPR 20.

% In this code, we apply S3COMP-C to the handwritten digit images. We 
% conduct experiment on the whole dataset, with feature extracted from a scattering network
% and projected to dimension 500.

% Instructions for running the code:
% - Download code for computing clustering accuracy. Go to
% http://www.cad.zju.edu.cn/home/dengcai/Data/Clustering.html and download
% bestMap.m and Hungarian.m. Alternatively, you can use your own function
% by redefining the function evalAccuracy.m
% - Download data. We use all images from MNIST and apply 
% scattering transform. Specifically, 
%   = Download MNIST traning image/label files from 
%     http://yann.lecun.com/exdb/mnist/ and put the data in folder MNIST/
%   = Download files for reading data (loadMNISTImages.m and 
%     loadMNISTLabels.m) from
%     http://ufldl.stanford.edu/wiki/index.php/MATLAB_Modules
%   = Download scattering transform package ScatNet (v0.2) from
%     http://www.di.ens.fr/data/software/
%     and install.
%   =vl_feat toolbox. Download and install the vl_feat toolbox from
%   http://www.vlfeat.org/. The code uses the functions vl_kdtreebuild, 
%   vl_kdtreequery and vl_kmeans from this toolbox. Alternatively, one can
%   also use these functionalities provided with Matlab. 
% - Run. 

%% Setting
% data
clear
nExperiment = 10;

buildRepresentation = @(data,dic,ind_1,lambda,C_final) S3COMP_C(data,dic,ind_1,lambda,C_final,10, 1e-6);
% second parameter is sparsity
% p=logspace(log10(0.05), log10(1), 20);
%% Load data
addpath('MNIST')
addpath('toolbox')
addpath('vlfeat-0.9.21')
run('vlfeat-0.9.21/toolbox/vl_setup')

if ~exist('MNIST_DATA', 'var')
    try
        load MNIST_SC.mat MNIST_SC_DATA MNIST_LABEL;
    catch
        MNIST_DATA = loadMNISTImages('train-images.idx3-ubyte');
        MNIST_LABEL = loadMNISTLabels('train-labels.idx1-ubyte');
        MNIST_SC_DATA = SCofDigits(MNIST_DATA);
        save MNIST_SC.mat MNIST_DATA MNIST_LABEL MNIST_SC_DATA;
    end
%    MNIST_DATA = MNIST_SC_DATA;
end
% load test images
if ~exist('MNIST_TDATA', 'var')
    try
        load MNIST_TSC.mat MNIST_SC_TDATA MNIST_TLABEL;
    catch
        MNIST_TDATA = loadMNISTImages('t10k-images.idx3-ubyte');
        MNIST_TLABEL = loadMNISTLabels('t10k-labels.idx1-ubyte');
        MNIST_SC_TDATA = SCofDigits(MNIST_TDATA);
        save MNIST_TSC.mat MNIST_TDATA MNIST_TLABEL MNIST_SC_TDATA;
    end
%    MNIST_TDATA = MNIST_SC_TDATA;
end
%load('MNIST_TSC.mat');
MNIST_DATA= [MNIST_SC_DATA MNIST_SC_TDATA];
MNIST_LABEL= [MNIST_LABEL; MNIST_TLABEL];
clear  MNIST_SC_DATA MNIST_SC_TDATA MNIST_TDATA MNIST_TLABEL


[~, N] = size(MNIST_DATA);
s=MNIST_LABEL;
%     fprintf('Dimension reduction...\n')
X = dimReduction_PCA(MNIST_DATA, 500);
%% Test begin
delta=0.8;  
T=10;
lambda=0.4;
results = zeros(length(delta),length(lambda),4);
% CoreNum=10;
% parpool ('local',CoreNum);
maxiter=3;
for ii=1:length(delta)
    delta_1=delta(ii);
    for jj=1:length(lambda)
        results_1=zeros(nExperiment, 4);
        for iExperiment = 1:nExperiment
            nCluster = 10;
            C=cell(1,T+1);
            count=zeros(1,N);
            idxcell=cell(1,T+1);
            for t=1:T
                a=rand(1,N);
                a=a>delta_1;
                idx=find(a>0);
                count(idx)=count(idx)+1;
                idxcell{t}=idx;
            end
            if all(count)
                T_all=T;
            else
                id1=find(count<1);
                a=rand(1,N);
                a=a>delta_1;
                idx=find(a>0);
                idx=union(idx,id1);
                count(idx)=count(idx)+1;
                idxcell{T+1}=idx;
                T_all=T+1;
            end 
            C_final=sparse(N,N);
            for iter=1:maxiter
                fprintf('iter:%d',iter);
                for t=1:T_all
                    fprintf('t:%d',t);
                    C_final_t=C_final(idxcell{t},:);
                    C{t}=buildRepresentation(X,X(:,idxcell{t})*(1/(1-delta_1)),idxcell{t},lambda(jj),C_final_t);
                end
                R=sparse(N,N);
                for t=1:T_all
                    R=R+C{t};
                end
                C_final=R/T_all;
            end
            C_final(1:N+1:end) = 0;
            A = abs(C_final) + abs(C_final)';
            groups=SpectralClustering3(A, 11, 'Eig_Solver', 'eigs');     
           % groups = SpectralClustering(A, nCluster, 'Eig_Solver', 'eigs','Kmeans_Solver', 'vl_kmeans');  


            % Evaluation
            [perc, vec] = evalSSR_perc( C_final, s );
            ssr = evalSSR_error( C_final, s );   
            conn = evalConn_mean( A, s);
            accr  = evalAccuracy(s, groups);

            dataformat = '%d-th experiment: perc = %f, ssr = %f, conn = %f, accr = %f\n';
            dataValue = [iExperiment, perc, ssr, conn, accr];
            fprintf(dataformat, dataValue);
            results_1(iExperiment, :) = dataValue(2:end);  
        end
        tmp_re=mean(results_1, 1);
        results(ii,jj,:)=tmp_re;
        dataformat = 'delta=%f,T=%f,lambda=%f,maxiter=%d: perc = %f, ssr = %f, conn = %f, accr = %f\n';
        dataValue = [delta_1,T,lambda(jj),maxiter,tmp_re];
        fprintf(dataformat, dataValue);
        fprintf('\n');
    end
end
% delete(gcp);